@@ -17,9 +17,8 @@
 
 use super::{
     data::{into_buffers, new_buffers},
-    ArrayData, ArrayDataBuilder,
+    ArrayData, ArrayDataBuilder, OffsetSizeTrait,
 };
-use crate::array::StringOffsetSizeTrait;
 use crate::{
     buffer::MutableBuffer,
     datatypes::DataType,
@@ -31,10 +30,12 @@ use std::mem;
 
 mod boolean;
 mod fixed_binary;
+mod fixed_size_list;
 mod list;
 mod null;
 mod primitive;
 mod structure;
+mod union;
 mod utils;
 mod variable_size;
 
@@ -85,7 +86,7 @@ impl<'a> _MutableArrayData<'a> {
             .child_data(child_data);
         if self.null_count > 0 {
             array_data_builder =
-                array_data_builder.null_bit_buffer(self.null_buffer.into());
+                array_data_builder.null_bit_buffer(Some(self.null_buffer.into()));
         }
 
         array_data_builder
@@ -266,17 +267,17 @@ fn build_extend(array: &ArrayData) -> Extend {
         DataType::LargeUtf8 | DataType::LargeBinary => {
             variable_size::build_extend::<i64>(array)
         }
-        DataType::List(_) => list::build_extend::<i32>(array),
+        DataType::Map(_, _) | DataType::List(_) => list::build_extend::<i32>(array),
         DataType::LargeList(_) => list::build_extend::<i64>(array),
         DataType::Dictionary(_, _) => unreachable!("should use build_extend_dictionary"),
         DataType::Struct(_) => structure::build_extend(array),
         DataType::FixedSizeBinary(_) => fixed_binary::build_extend(array),
         DataType::Float16 => primitive::build_extend::<f16>(array),
-        /*
-        DataType::FixedSizeList(_, _) => {}
-        DataType::Union(_) => {}
-        */
-        _ => todo!("Take and filter operations still not supported for this datatype"),
+        DataType::FixedSizeList(_, _) => fixed_size_list::build_extend(array),
+        DataType::Union(_, _, mode) => match mode {
+            UnionMode::Sparse => union::build_extend_sparse(array),
+            UnionMode::Dense => union::build_extend_dense(array),
+        },
     }
 }
 
@@ -307,7 +308,7 @@ fn build_extend_nulls(data_type: &DataType) -> ExtendNulls {
         DataType::Interval(IntervalUnit::MonthDayNano) => primitive::extend_nulls::<i128>,
         DataType::Utf8 | DataType::Binary => variable_size::extend_nulls::<i32>,
         DataType::LargeUtf8 | DataType::LargeBinary => variable_size::extend_nulls::<i64>,
-        DataType::List(_) => list::extend_nulls::<i32>,
+        DataType::Map(_, _) | DataType::List(_) => list::extend_nulls::<i32>,
         DataType::LargeList(_) => list::extend_nulls::<i64>,
         DataType::Dictionary(child_data_type, _) => match child_data_type.as_ref() {
             DataType::UInt8 => primitive::extend_nulls::<u8>,
@@ -323,22 +324,22 @@ fn build_extend_nulls(data_type: &DataType) -> ExtendNulls {
         DataType::Struct(_) => structure::extend_nulls,
         DataType::FixedSizeBinary(_) => fixed_binary::extend_nulls,
         DataType::Float16 => primitive::extend_nulls::<f16>,
-        /*
-        DataType::FixedSizeList(_, _) => {}
-        DataType::Union(_) => {}
-        */
-        _ => todo!("Take and filter operations still not supported for this datatype"),
+        DataType::FixedSizeList(_, _) => fixed_size_list::extend_nulls,
+        DataType::Union(_, _, mode) => match mode {
+            UnionMode::Sparse => union::extend_nulls_sparse,
+            UnionMode::Dense => union::extend_nulls_dense,
+        },
     })
 }
 
-fn preallocate_offset_and_binary_buffer<Offset: StringOffsetSizeTrait>(
+fn preallocate_offset_and_binary_buffer<Offset: OffsetSizeTrait>(
     capacity: usize,
     binary_size: usize,
 ) -> [MutableBuffer; 2] {
     // offsets
     let mut buffer = MutableBuffer::new((1 + capacity) * mem::size_of::<Offset>());
     // safety: `unsafe` code assumes that this buffer is initialized with one element
-    if Offset::is_large() {
+    if Offset::IS_LARGE {
         buffer.push(0i64);
     } else {
         buffer.push(0i32)
@@ -452,7 +453,7 @@ impl<'a> MutableArrayData<'a> {
             | DataType::LargeBinary
             | DataType::Interval(_)
             | DataType::FixedSizeBinary(_) => vec![],
-            DataType::List(_) | DataType::LargeList(_) => {
+            DataType::Map(_, _) | DataType::List(_) | DataType::LargeList(_) => {
                 let childs = arrays
                     .iter()
                     .map(|array| &array.child_data()[0])
@@ -516,9 +517,22 @@ impl<'a> MutableArrayData<'a> {
                     })
                     .collect::<Vec<_>>(),
             },
-            _ => {
-                todo!("Take and filter operations still not supported for this datatype")
+            DataType::FixedSizeList(_, _) => {
+                let childs = arrays
+                    .iter()
+                    .map(|array| &array.child_data()[0])
+                    .collect::<Vec<_>>();
+                vec![MutableArrayData::new(childs, use_nulls, array_capacity)]
             }
+            DataType::Union(fields, _, _) => (0..fields.len())
+                .map(|i| {
+                    let child_arrays = arrays
+                        .iter()
+                        .map(|array| &array.child_data()[i])
+                        .collect::<Vec<_>>();
+                    MutableArrayData::new(child_arrays, use_nulls, array_capacity)
+                })
+                .collect::<Vec<_>>(),
         };
 
         // Get the dictionary if any, and if it is a concatenation of multiple
@@ -683,8 +697,8 @@ mod tests {
         array::{
             Array, ArrayData, ArrayRef, BooleanArray, DictionaryArray,
             FixedSizeBinaryArray, Int16Array, Int16Type, Int32Array, Int64Array,
-            Int64Builder, ListBuilder, NullArray, PrimitiveBuilder, StringArray,
-            StringDictionaryBuilder, StructArray, UInt8Array,
+            Int64Builder, ListBuilder, MapBuilder, NullArray, PrimitiveBuilder,
+            StringArray, StringDictionaryBuilder, StructArray, UInt8Array,
         },
         buffer::Buffer,
         datatypes::Field,
@@ -1238,7 +1252,6 @@ mod tests {
             DataType::List(Box::new(Field::new("item", DataType::Int64, true))),
             8,
             None,
-            None,
             0,
             vec![list_value_offsets],
             vec![expected_int_array.data().clone()],
@@ -1319,7 +1332,6 @@ mod tests {
         let expected_list_data = ArrayData::try_new(
             DataType::List(Box::new(Field::new("item", DataType::Int64, true))),
             12,
-            None,
             Some(Buffer::from(&[0b11011011, 0b1110])),
             0,
             vec![list_value_offsets],
@@ -1331,6 +1343,156 @@ mod tests {
         Ok(())
     }
 
+    #[test]
+    fn test_map_nulls_append() -> Result<()> {
+        let mut builder = MapBuilder::<Int64Builder, Int64Builder>::new(
+            None,
+            Int64Builder::new(32),
+            Int64Builder::new(32),
+        );
+        builder.keys().append_slice(&[1, 2, 3])?;
+        builder.values().append_slice(&[1, 2, 3])?;
+        builder.append(true)?;
+        builder.keys().append_slice(&[4, 5])?;
+        builder.values().append_slice(&[4, 5])?;
+        builder.append(true)?;
+        builder.append(false)?;
+        builder
+            .keys()
+            .append_slice(&[6, 7, 8, 100, 101, 9, 10, 11])?;
+        builder.values().append_slice(&[6, 7, 8])?;
+        builder.values().append_null()?;
+        builder.values().append_null()?;
+        builder.values().append_slice(&[9, 10, 11])?;
+        builder.append(true)?;
+
+        let a = builder.finish();
+        let a = a.data();
+
+        let mut builder = MapBuilder::<Int64Builder, Int64Builder>::new(
+            None,
+            Int64Builder::new(32),
+            Int64Builder::new(32),
+        );
+
+        builder.keys().append_slice(&[12, 13])?;
+        builder.values().append_slice(&[12, 13])?;
+        builder.append(true)?;
+        builder.append(false)?;
+        builder.append(true)?;
+        builder.keys().append_slice(&[100, 101, 14, 15])?;
+        builder.values().append_null()?;
+        builder.values().append_null()?;
+        builder.values().append_slice(&[14, 15])?;
+        builder.append(true)?;
+
+        let b = builder.finish();
+        let b = b.data();
+        let c = b.slice(1, 2);
+        let d = b.slice(2, 2);
+
+        let mut mutable = MutableArrayData::new(vec![a, b, &c, &d], false, 10);
+
+        mutable.extend(0, 0, a.len());
+        mutable.extend(1, 0, b.len());
+        mutable.extend(2, 0, c.len());
+        mutable.extend(3, 0, d.len());
+        let result = mutable.freeze();
+
+        let expected_key_array = Int64Array::from(vec![
+            Some(1),
+            Some(2),
+            Some(3),
+            Some(4),
+            Some(5),
+            Some(6),
+            Some(7),
+            Some(8),
+            Some(100),
+            Some(101),
+            Some(9),
+            Some(10),
+            Some(11),
+            // second array
+            Some(12),
+            Some(13),
+            Some(100),
+            Some(101),
+            Some(14),
+            Some(15),
+            // slice(1, 2) results in no values added
+            Some(100),
+            Some(101),
+            Some(14),
+            Some(15),
+        ]);
+
+        let expected_value_array = Int64Array::from(vec![
+            Some(1),
+            Some(2),
+            Some(3),
+            Some(4),
+            Some(5),
+            Some(6),
+            Some(7),
+            Some(8),
+            None,
+            None,
+            Some(9),
+            Some(10),
+            Some(11),
+            // second array
+            Some(12),
+            Some(13),
+            None,
+            None,
+            Some(14),
+            Some(15),
+            // slice(1, 2) results in no values added
+            None,
+            None,
+            Some(14),
+            Some(15),
+        ]);
+
+        let expected_entry_array = StructArray::from(vec![
+            (
+                Field::new("keys", DataType::Int64, false),
+                Arc::new(expected_key_array) as ArrayRef,
+            ),
+            (
+                Field::new("values", DataType::Int64, true),
+                Arc::new(expected_value_array) as ArrayRef,
+            ),
+        ]);
+
+        let map_offsets =
+            Buffer::from_slice_ref(&[0, 3, 5, 5, 13, 15, 15, 15, 19, 19, 19, 19, 23]);
+
+        let expected_list_data = ArrayData::try_new(
+            DataType::Map(
+                Box::new(Field::new(
+                    "entries",
+                    DataType::Struct(vec![
+                        Field::new("keys", DataType::Int64, false),
+                        Field::new("values", DataType::Int64, true),
+                    ]),
+                    false,
+                )),
+                false,
+            ),
+            12,
+            Some(Buffer::from(&[0b11011011, 0b1110])),
+            0,
+            vec![map_offsets],
+            vec![expected_entry_array.data().clone()],
+        )
+        .unwrap();
+        assert_eq!(result, expected_list_data);
+
+        Ok(())
+    }
+
     #[test]
     fn test_list_of_strings_append() -> Result<()> {
         // [["alpha", "beta", None]]
@@ -1392,7 +1554,6 @@ mod tests {
             DataType::List(Box::new(Field::new("item", DataType::Utf8, true))),
             6,
             None,
-            None,
             0,
             vec![list_value_offsets],
             vec![expected_string_array.data().clone()],
