@@ -496,7 +496,7 @@ fn array_from_json(
                 .offset(0)
                 .add_buffer(Buffer::from(&offsets.to_byte_slice()))
                 .add_child_data(child_array.data().clone())
-                .null_bit_buffer(null_buf)
+                .null_bit_buffer(Some(null_buf))
                 .build()
                 .unwrap();
             Ok(Arc::new(ListArray::from(list_data)))
@@ -524,7 +524,7 @@ fn array_from_json(
                 .offset(0)
                 .add_buffer(Buffer::from(&offsets.to_byte_slice()))
                 .add_child_data(child_array.data().clone())
-                .null_bit_buffer(null_buf)
+                .null_bit_buffer(Some(null_buf))
                 .build()
                 .unwrap();
             Ok(Arc::new(LargeListArray::from(list_data)))
@@ -540,7 +540,7 @@ fn array_from_json(
             let list_data = ArrayData::builder(field.data_type().clone())
                 .len(json_col.count)
                 .add_child_data(child_array.data().clone())
-                .null_bit_buffer(null_buf)
+                .null_bit_buffer(Some(null_buf))
                 .build()
                 .unwrap();
             Ok(Arc::new(FixedSizeListArray::from(list_data)))
@@ -550,7 +550,7 @@ fn array_from_json(
             let null_buf = create_null_buf(&json_col);
             let mut array_data = ArrayData::builder(field.data_type().clone())
                 .len(json_col.count)
-                .null_bit_buffer(null_buf);
+                .null_bit_buffer(Some(null_buf));
 
             for (field, col) in fields.iter().zip(json_col.children.unwrap()) {
                 let array = array_from_json(field, col, dictionaries)?;
@@ -625,46 +625,20 @@ fn array_from_json(
                 .len(json_col.count)
                 .add_buffer(Buffer::from(&offsets.to_byte_slice()))
                 .add_child_data(child_array.data().clone())
-                .null_bit_buffer(null_buf)
+                .null_bit_buffer(Some(null_buf))
                 .build()
                 .unwrap();
 
             let array = MapArray::from(array_data);
             Ok(Arc::new(array))
         }
-        DataType::Union(fields, _) => {
-            let field_type_ids = fields
-                .iter()
-                .enumerate()
-                .into_iter()
-                .map(|(idx, f)| {
-                    (
-                        f.metadata()
-                            .and_then(|m| m.get("type_id"))
-                            .unwrap()
-                            .parse::<i8>()
-                            .unwrap(),
-                        idx,
-                    )
-                })
-                .collect::<HashMap<_, _>>();
-
+        DataType::Union(fields, field_type_ids, _) => {
             let type_ids = if let Some(type_id) = json_col.type_id {
                 type_id
-                    .iter()
-                    .map(|t| {
-                        if field_type_ids.contains_key(t) {
-                            Ok(*(field_type_ids.get(t).unwrap()) as i8)
-                        } else {
-                            Err(ArrowError::JsonError(format!(
-                                "Unable to find type id {:?}",
-                                t
-                            )))
-                        }
-                    })
-                    .collect::<Result<_>>()?
             } else {
-                vec![]
+                return Err(ArrowError::JsonError(
+                    "Cannot find expected type_id in json column".to_string(),
+                ));
             };
 
             let offset: Option<Buffer> = json_col.offset.map(|offsets| {
@@ -680,6 +654,7 @@ fn array_from_json(
             }
 
             let array = UnionArray::try_new(
+                field_type_ids,
                 Buffer::from(&type_ids.to_byte_slice()),
                 offset,
                 children,
@@ -738,7 +713,7 @@ fn dictionary_array_from_json(
             let dict_data = ArrayData::builder(field.data_type().clone())
                 .len(keys.len())
                 .add_buffer(keys.data().buffers()[0].clone())
-                .null_bit_buffer(null_buf)
+                .null_bit_buffer(Some(null_buf))
                 .add_child_data(values.data().clone())
                 .build()
                 .unwrap();
