@@ -222,7 +222,7 @@ impl<OffsetSize: OffsetSizeTrait> ArrayReader for ListArrayReader<OffsetSize> {
 
         if let Some(mut builder) = validity {
             assert_eq!(builder.len(), list_offsets.len() - 1);
-            data_builder = data_builder.null_bit_buffer(builder.finish())
+            data_builder = data_builder.null_bit_buffer(Some(builder.finish()))
         }
 
         let list_data = unsafe { data_builder.build_unchecked() };
@@ -246,7 +246,7 @@ mod tests {
     use crate::arrow::array_reader::build_array_reader;
     use crate::arrow::array_reader::list_array::ListArrayReader;
     use crate::arrow::array_reader::test_util::InMemoryArrayReader;
-    use crate::arrow::{parquet_to_arrow_schema, ArrowWriter};
+    use crate::arrow::{parquet_to_arrow_schema, ArrowWriter, ProjectionMask};
     use crate::file::properties::WriterProperties;
     use crate::file::reader::{FileReader, SerializedFileReader};
     use crate::schema::parser::parse_message_type;
@@ -327,7 +327,7 @@ mod tests {
             .len(10)
             .add_buffer(offsets)
             .add_child_data(leaf.data().clone())
-            .null_bit_buffer(Buffer::from([0b11111101, 0b00000010]))
+            .null_bit_buffer(Some(Buffer::from([0b11111101, 0b00000010])))
             .build()
             .unwrap();
 
@@ -345,7 +345,7 @@ mod tests {
             .len(4)
             .add_buffer(offsets)
             .add_child_data(l2)
-            .null_bit_buffer(Buffer::from([0b00001101]))
+            .null_bit_buffer(Some(Buffer::from([0b00001101])))
             .build()
             .unwrap();
 
@@ -564,7 +564,7 @@ mod tests {
             .set_max_row_group_size(200)
             .build();
 
-        let mut writer = ArrowWriter::try_new(
+        let writer = ArrowWriter::try_new(
             file.try_clone().unwrap(),
             Arc::new(arrow_schema),
             Some(props),
@@ -582,18 +582,30 @@ mod tests {
         )
         .unwrap();
 
+        let schema = file_metadata.schema_descr_ptr();
+        let mask = ProjectionMask::leaves(&schema, vec![0]);
+
         let mut array_reader = build_array_reader(
-            file_reader.metadata().file_metadata().schema_descr_ptr(),
-            Arc::new(arrow_schema.clone()),
-            vec![0usize].into_iter(),
+            schema,
+            Arc::new(arrow_schema),
+            mask,
             Box::new(file_reader),
         )
         .unwrap();
 
         let batch = array_reader.next_batch(100).unwrap();
+        assert_eq!(batch.data_type(), array_reader.get_data_type());
         assert_eq!(
             batch.data_type(),
-            &ArrowType::Struct(arrow_schema.fields().clone())
+            &ArrowType::Struct(vec![Field::new(
+                "table_info",
+                ArrowType::List(Box::new(Field::new(
+                    "table_info",
+                    ArrowType::Struct(vec![Field::new("name", ArrowType::Binary, false)]),
+                    false
+                ))),
+                false
+            )])
         );
         assert_eq!(batch.len(), 0);
     }
